==> ninix-aya-4.99.4.tgz: ninix-aya-4.99.4/lib/ninix/dll/ssu.py <==
# -*- coding: utf-8 -*-
#
#  ssu.py - a ssu compatible Saori module for ninix
#  Copyright (C) 2003-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#

# TODO:
# - if, switch等の引数計算(calcを使用)

import codecs
import logging
import re

from ninix.dll import SAORI


class Saori(SAORI):

    def __init__(self):
        SAORI.__init__(self)
        self.function = {'is_empty':      [self.ssu_is_empty,      [0, 1]],
                         'is_digit':      [self.ssu_is_digit,      [1]],
                         'is_alpha':      [self.ssu_is_alpha,      [1]],
                         'iflist':        [self.ssu_iflist,        [None]],
                         'length':        [self.ssu_length,        [1]],
                         'zen2han':       [self.ssu_zen2han,       [1]],
                         'han2zen':       [self.ssu_han2zen,       [1]],
                         'kata2hira,':    [self.ssu_kata2hira,     [1]],
                         'hira2kata':     [self.ssu_hira2kata,     [1]],
                         'sprintf':       [self.ssu_sprintf,       [None]],
                         'calc':          [self.ssu_calc,          [1]],
                         'calc_float':    [self.ssu_calc_float,    [1]],
                         'compare_tail':  [self.ssu_compare_tail,  [2]],
                         'compare_head':  [self.ssu_compare_head,  [2]],
                         'compare':       [self.ssu_compare,       [2]],
                         'count':         [self.ssu_count,         [2]],
                         'erase_first':   [self.ssu_erase_first,   [2]],
                         'erase':         [self.ssu_erase,         [2]],
                         'replace':       [self.ssu_replace,       [3]],
                         'replace_first': [self.ssu_replace_first, [3]],
                         'split':         [self.ssu_split,         [1, 2, 3]],
                         'substr':        [self.ssu_substr,        [3]],
                         'nswitch':       [self.ssu_nswitch,       [None]],
                         'switch':        [self.ssu_switch,        [None]],
                         'if':            [self.ssu_if,            [2, 3]],
                         'unless':        [self.ssu_unless,        [2, 3]],}

    def request(self, req):
        req_type, argument, charset = self.evaluate_request(req)
        if not req_type:
            return self.RESPONSE[400]
        elif req_type == 'GET Version':
            return self.RESPONSE[204]
        elif req_type == 'EXECUTE':
            result = self.execute(argument)
        else:
            return self.RESPONSE[400]

    def execute(self, args):
        if args[0] not in self.function:
            return self.RESPONSE[400]
        name = args[0]
        args = args[1:]
        if self.function[name][1] == [None]:
            pass
        elif len(args) not in self.function[name][1]:
            return self.RESPONSE[400]
        self.value = []
        result = self.function[name][0](args)
        if result is not None and result != '':
            s = 'SAORI/1.0 200 OK\r\nResult: {0}\r\n'.format(result)
            if self.value:
                for i in range(len(self.value)):
                    s = ''.join((s, 'Value{0:d}: {1}\r\n'.format(i, self.value[i])))
            s = ''.join((s, 'Charset: {0}\r\n'.format(charset)))
            s = ''.join((s, '\r\n'))
            return s.encode(charset)
        else:
            return self.RESPONSE[204]

    def ssu_is_empty(self, args):
        return 1 if not args else 0

    def ssu_is_digit(self, args):
        s = args[0]
        return 1 if s.isdigit() else 0

    def ssu_is_alpha(self, args):
        s = args[0]
        return 1 if s.isalpha() else 0

    def ssu_length(self, args):
        s = args[0]
        return len(s)

    def ssu_substr(self, args):
        s = args[0]
        if self.ssu_is_digit([args[1]]):
            start = int(self.ssu_zen2han([args[1]]))
            if start > len(s):
                return ''
        else:
            return ''
        if len(args) == 2:
            end = len(s)
        elif len(args) == 3 and self.ssu_is_digit([args[2]]):
            end = start + int(self.ssu_zen2han([args[2]]))
        else:
            return ''
        return s[start:end]

    def ssu_sprintf(self, args): ## FIXME
        for i in range(1, len(args)):
            if self.ssu_is_digit([args[i]]):
                args[i] = int(self.ssu_zen2han([args[i]])[0])
        try:
            return args[0] % tuple(args[1:])
        except:
            return ''

    ZEN = {'０': '0', '１': '1', '２': '2', '３': '3', '４': '4',
           '５': '5', '６': '6', '７': '7', '８': '8', '９': '9',
           '．': '.', '＋': '+', '−': '-',}
    HAN = {'0': '０', '1': '１', '2': '２', '3': '３', '4': '４',
           '5': '５', '6': '６', '7': '７', '8': '８', '9': '９',
           '.': '．', '+': '＋', '-': '−',}

    def ssu_zen2han(self, args):
        s = args[0]
        buf = ''
        for i in range(len(s)):
            c = s[i]
            if c in self.ZEN:
                buf = ''.join((buf, self.ZEN[c]))
            else:
                buf = ''.join((buf, s[i]))
        return buf

    def ssu_han2zen(self, args):
        s = args[0]
        buf = ''
        for i in range(len(s)):
            c = s[i]
            if c in self.HAN:
                buf = ''.join((buf, self.HAN[c]))
            else:
                buf = ''.join((buf, s[i]))
        return buf

    re_condition = re.compile(r'(>|<|==|>=|<=|!=|＞|＜|＞＝|＜＝|！＝|＝＝)')

    def eval_condition(self, left, ope, right):
        if self.ssu_is_digit([left]) and \
           self.ssu_is_digit([right]):
            left = float(self.ssu_zen2han([left]))
            right = float(self.ssu_zen2han([right]))
        elif ope in ['>', '＞', '>=', '＞＝', '<', '＜', '<=', '＜＝']:
            left = len(left)
            right = len(right)
        result = False
        if ope in ['>', '＞']:
            result = left > right
        elif ope in ['>=', '＞＝']:
            result = left >= right
        elif ope in ['<', '＜']:
            result = left < right
        elif ope in ['<=', '＜＝']:
            result = left <= right
        elif ope in ['==', '＝＝']:
            result = left == right
        elif ope in ['!=', '！＝']:
            result = left != right
        else:
            pass # 'should not reach here'
        return result

    def ssu_if(self, args):
        condition = args[0]
        match = self.re_condition.search(condition)
        if match is not None:
            left = condition[:match.start()]
            ope = match.group(1)
            right = condition[match.end():]
            result = self.eval_condition(left, ope, right)
            if result:
                return args[1]
        return args[2] if len(args) == 3 else ''

    def ssu_unless(self, args):
        condition = args[0]
        match = self.re_condition.search(condition)
        if match is not None:
            left = condition[:match.start()]
            ope = match.group(1)
            right = condition[match.end():]
            result = self.eval_condition(left, ope, right)
            if not result:
                return args[1]
        return args[2] if len(args) == 3 else ''

    def ssu_iflist(self, args):
        left = args[0]
        i = 1
        while 1:
            if len(args[i:]) < 2:
                break
            ope_right = args[i]
            match = self.re_condition.search(ope_right)
            if match is not None:
                ope = match.group(1)
                right = ope_right[match.end():]
                result = self.eval_condition(left, ope, right)
                if result:
                    return args[i + 1]
            i += 2
        return ''

    def ssu_nswitch(self, args):
        num = args[0]
        if self.ssu_is_digit([num]):
            num = int(self.ssu_zen2han([num]))
            if 0 < num < len(args):
                return args[num]
        return ''

    def ssu_count(self, args):
        return args[0].count(args[1])

    def ssu_compare(self, args):
        return 1 if args[0] == args[1] else 0

    def ssu_compare_head(self, args):
        s0 = args[0]
        s1 = args[1]
        return 1 if s1.startswith(s0) else 0

    def ssu_compare_tail(self, args):
        s0 = args[0]
        s1 = args[1]
        return 1 if s1.endswith(s0) else 0

    def ssu_erase(self, args):
        return args[0].replace(args[1], '')

    def ssu_erase_first(self, args):
        return args[0].replace(args[1], '', 1)

    def ssu_replace(self, args):
        return args[0].replace(args[1], args[2])

    def ssu_replace_first(self, args):
        return args[0].replace(args[1], args[2], 1)

    def ssu_split(self, args):
        s0 = args[0]
        if len(args) >= 2:
            s1 = args[1]
        else:
            s1 = ' ' ## FIXME
        if len(args) == 3:
            num = args[0]
            if self.ssu_is_digit([num]):
                num = int(self.ssu_zen2han([num]))
            value_list = s0.split(s1, num)
        else:
            value_list = s0.split(s1)
        self.value = value_list
        return len(value_list)

    def ssu_switch(self, args):
        left = args[0]
        i = 1
        while 1:
            if len(args[i:]) < 2:
                break
            right = args[i]
            if left == right:
                return args[i + 1]
            i += 2
        return ''

    def ssu_kata2hira(self, args): ## FIXME
        return ''

    def ssu_hira2kata(self, args): ## FIXME
        return ''

    def ssu_calc(self, args): ## FIXME
        return 0

    def ssu_calc_float(self, args): ## FIXME
        return 0

    def evaluate_request(self, req):
        req_type = None
        argument = []
        charset = 'CP932' # default
        header = req.splitlines()
        line = header.pop(0)
        if not line:
            return req_type, argument, charset
        line = str(line, charset).strip()
        if not line:
            return req_type, argument, charset
        for request in ['EXECUTE', 'GET Version']:
            if line.startswith(request):
                req_type = request
                break
        for line in header:
            line = str(line, charset).strip()
            if not line:
                continue
            if ':' not in line:
                continue
            key, value = [x.strip() for x in line.split(':', 1)]
            if key == 'Charset':
                charset = value
                try:
                    codecs.lookup(charset)
                except:
                    logging.warning('Unsupported charset {0}'.format(repr(charset)))
            if key.startswith('Argument'): ## FIXME
                argument.append(value)
            else:
                continue
        return req_type, argument, charset

==> ninix-aya-4.99.7.tgz: ninix-aya-4.99.7/lib/ninix/dll/ssu.py <==
# -*- coding: utf-8 -*-
#
#  ssu.py - a ssu compatible Saori module for ninix
#  Copyright (C) 2003-2012 by Shyouzou Sugitani <shy@users.sourceforge.jp>
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License (version 2) as
#  published by the Free Software Foundation.  It is distributed in the
#  hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
#  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE.  See the GNU General Public License for more details.
#

# TODO:
# - if, switch等の引数計算(calcを使用)

import codecs
import logging
import re

from ninix.dll import SAORI


class Saori(SAORI):

    def __init__(self):
        SAORI.__init__(self)
        self.function = {'is_empty':      [self.ssu_is_empty,      [0, 1]],
                         'is_digit':      [self.ssu_is_digit,      [1]],
                         'is_alpha':      [self.ssu_is_alpha,      [1]],
                         'iflist':        [self.ssu_iflist,        [None]],
                         'length':        [self.ssu_length,        [1]],
                         'zen2han':       [self.ssu_zen2han,       [1]],
                         'han2zen':       [self.ssu_han2zen,       [1]],
                         'kata2hira,':    [self.ssu_kata2hira,     [1]],
                         'hira2kata':     [self.ssu_hira2kata,     [1]],
                         'sprintf':       [self.ssu_sprintf,       [None]],
                         'calc':          [self.ssu_calc,          [1]],
                         'calc_float':    [self.ssu_calc_float,    [1]],
                         'compare_tail':  [self.ssu_compare_tail,  [2]],
                         'compare_head':  [self.ssu_compare_head,  [2]],
                         'compare':       [self.ssu_compare,       [2]],
                         'count':         [self.ssu_count,         [2]],
                         'erase_first':   [self.ssu_erase_first,   [2]],
                         'erase':         [self.ssu_erase,         [2]],
                         'replace':       [self.ssu_replace,       [3]],
                         'replace_first': [self.ssu_replace_first, [3]],
                         'split':         [self.ssu_split,         [1, 2, 3]],
                         'substr':        [self.ssu_substr,        [3]],
                         'nswitch':       [self.ssu_nswitch,       [None]],
                         'switch':        [self.ssu_switch,        [None]],
                         'if':            [self.ssu_if,            [2, 3]],
                         'unless':        [self.ssu_unless,        [2, 3]],}

    def request(self, req):
        req_type, argument, charset = self.evaluate_request(req)
        if not req_type:
            result = self.RESPONSE[400]
        elif req_type == 'GET Version':
            result = self.RESPONSE[204]
        elif req_type == 'EXECUTE':
            result = self.execute(argument, charset)
        else:
            result = self.RESPONSE[400]
        return result

    def execute(self, args, charset):
        if args[0] not in self.function:
            return self.RESPONSE[400]
        name = args[0]
        args = args[1:]
        if self.function[name][1] == [None]:
            pass
        elif len(args) not in self.function[name][1]:
            return self.RESPONSE[400]
        self.value = []
        result = self.function[name][0](args)
        if result is not None and result != '':
            s = 'SAORI/1.0 200 OK\r\nResult: {0}\r\n'.format(result)
            if self.value:
                for i in range(len(self.value)):
                    s = ''.join((s, 'Value{0:d}: {1}\r\n'.format(i, self.value[i])))
            s = ''.join((s, 'Charset: {0}\r\n'.format(charset)))
            s = ''.join((s, '\r\n'))
            return s.encode(charset)
        else:
            return self.RESPONSE[204]

    def ssu_is_empty(self, args):
        return 1 if not args else 0

    def ssu_is_digit(self, args):
        s = args[0]
        return 1 if s.isdigit() else 0

    def ssu_is_alpha(self, args):
        s = args[0]
        return 1 if s.isalpha() else 0

    def ssu_length(self, args):
        s = args[0]
        return len(s)

    def ssu_substr(self, args):
        s = args[0]
        if self.ssu_is_digit([args[1]]):
            start = int(self.ssu_zen2han([args[1]]))
            if start > len(s):
                return ''
        else:
            return ''
        if len(args) == 2:
            end = len(s)
        elif len(args) == 3 and self.ssu_is_digit([args[2]]):
            end = start + int(self.ssu_zen2han([args[2]]))
        else:
            return ''
        return s[start:end]

    def ssu_sprintf(self, args): ## FIXME
        for i in range(1, len(args)):
            if self.ssu_is_digit([args[i]]):
                args[i] = int(self.ssu_zen2han([args[i]])[0])
        try:
            return args[0] % tuple(args[1:])
        except:
            return ''

    ZEN = {'０': '0', '１': '1', '２': '2', '３': '3', '４': '4',
           '５': '5', '６': '6', '７': '7', '８': '8', '９': '9',
           '．': '.', '＋': '+', '−': '-',}
    HAN = {'0': '０', '1': '１', '2': '２', '3': '３', '4': '４',
           '5': '５', '6': '６', '7': '７', '8': '８', '9': '９',
           '.': '．', '+': '＋', '-': '−',}

    def ssu_zen2han(self, args):
        s = args[0]
        buf = ''
        for i in range(len(s)):
            c = s[i]
            if c in self.ZEN:
                buf = ''.join((buf, self.ZEN[c]))
            else:
                buf = ''.join((buf, s[i]))
        return buf

    def ssu_han2zen(self, args):
        s = args[0]
        buf = ''
        for i in range(len(s)):
            c = s[i]
            if c in self.HAN:
                buf = ''.join((buf, self.HAN[c]))
            else:
                buf = ''.join((buf, s[i]))
        return buf

    re_condition = re.compile(r'(>|<|==|>=|<=|!=|＞|＜|＞＝|＜＝|！＝|＝＝)')

    def eval_condition(self, left, ope, right):
        if self.ssu_is_digit([left]) and \
           self.ssu_is_digit([right]):
            left = float(self.ssu_zen2han([left]))
            right = float(self.ssu_zen2han([right]))
        elif ope in ['>', '＞', '>=', '＞＝', '<', '＜', '<=', '＜＝']:
            left = len(left)
            right = len(right)
        result = False
        if ope in ['>', '＞']:
            result = left > right
        elif ope in ['>=', '＞＝']:
            result = left >= right
        elif ope in ['<', '＜']:
            result = left < right
        elif ope in ['<=', '＜＝']:
            result = left <= right
        elif ope in ['==', '＝＝']:
            result = left == right
        elif ope in ['!=', '！＝']:
            result = left != right
        else:
            pass # 'should not reach here'
        return result

    def ssu_if(self, args):
        condition = args[0]
        match = self.re_condition.search(condition)
        if match is not None:
            left = condition[:match.start()]
            ope = match.group(1)
            right = condition[match.end():]
            result = self.eval_condition(left, ope, right)
            if result:
                return args[1]
        return args[2] if len(args) == 3 else ''

    def ssu_unless(self, args):
        condition = args[0]
        match = self.re_condition.search(condition)
        if match is not None:
            left = condition[:match.start()]
            ope = match.group(1)
            right = condition[match.end():]
            result = self.eval_condition(left, ope, right)
            if not result:
                return args[1]
        return args[2] if len(args) == 3 else ''

    def ssu_iflist(self, args):
        left = args[0]
        i = 1
        while 1:
            if len(args[i:]) < 2:
                break
            ope_right = args[i]
            match = self.re_condition.search(ope_right)
            if match is not None:
                ope = match.group(1)
                right = ope_right[match.end():]
                result = self.eval_condition(left, ope, right)
                if result:
                    return args[i + 1]
            i += 2
        return ''

    def ssu_nswitch(self, args):
        num = args[0]
        if self.ssu_is_digit([num]):
            num = int(self.ssu_zen2han([num]))
            if 0 < num < len(args):
                return args[num]
        return ''

    def ssu_count(self, args):
        return args[0].count(args[1])

    def ssu_compare(self, args):
        return 1 if args[0] == args[1] else 0

    def ssu_compare_head(self, args):
        s0 = args[0]
        s1 = args[1]
        return 1 if s1.startswith(s0) else 0

    def ssu_compare_tail(self, args):
        s0 = args[0]
        s1 = args[1]
        return 1 if s1.endswith(s0) else 0

    def ssu_erase(self, args):
        return args[0].replace(args[1], '')

    def ssu_erase_first(self, args):
        return args[0].replace(args[1], '', 1)

    def ssu_replace(self, args):
        return args[0].replace(args[1], args[2])

    def ssu_replace_first(self, args):
        return args[0].replace(args[1], args[2], 1)

    def ssu_split(self, args):
        s0 = args[0]
        if len(args) >= 2:
            s1 = args[1]
        else:
            s1 = ' ' ## FIXME
        if len(args) == 3:
            num = args[0]
            if self.ssu_is_digit([num]):
                num = int(self.ssu_zen2han([num]))
            value_list = s0.split(s1, num)
        else:
            value_list = s0.split(s1)
        self.value = value_list
        return len(value_list)

    def ssu_switch(self, args):
        left = args[0]
        i = 1
        while 1:
            if len(args[i:]) < 2:
                break
            right = args[i]
            if left == right:
                return args[i + 1]
            i += 2
        return ''

    def ssu_kata2hira(self, args): ## FIXME
        return ''

    def ssu_hira2kata(self, args): ## FIXME
        return ''

    def ssu_calc(self, args): ## FIXME
        return 0

    def ssu_calc_float(self, args): ## FIXME
        return 0

    def evaluate_request(self, req):
        req_type = None
        argument = []
        charset = 'CP932' # default
        header = req.splitlines()
        line = header.pop(0)
        if not line:
            return req_type, argument, charset
        line = str(line, charset).strip()
        if not line:
            return req_type, argument, charset
        for request in ['EXECUTE', 'GET Version']:
            if line.startswith(request):
                req_type = request
                break
        for line in header:
            line = str(line, charset).strip()
            if not line:
                continue
            if ':' not in line:
                continue
            key, value = [x.strip() for x in line.split(':', 1)]
            if key == 'Charset':
                charset = value
                try:
                    codecs.lookup(charset)
                except:
                    logging.warning('Unsupported charset {0}'.format(repr(charset)))
            if key.startswith('Argument'): ## FIXME
                argument.append(value)
            else:
                continue
        return req_type, argument, charset
